#!/usr/bin/perl
################################################################
#
# WEKO移行用フォーマットへの変換プログラムメイン
#
# パラメタを受け取り、リポジトリの種別に応じた処理を実行する。
#
################################################################

use strict;

use lib './include';
use common;
use Jcode;

#################################################################
# パラメタチェック
#################################################################
if(@ARGV < 2){
	print "[ERROR]パラメタが指定されていません。\n";
	print "usage: perl output2weko.pl リポジトリ種別 パラメタファイル\n";
	print "例: perl output2weko.pl dspace ./param.ini\n";
	exit(0);
}

#---------------------
# リポジトリ種別
#---------------------
my $kind = $ARGV[0];
#---------------------
# パラメタファイル
#---------------------
my $pfile = $ARGV[1];

#################################################################
# パラメタチェック
#################################################################
my %argments = ();	# パラメタ格納用hash
# パラメタを%argmentsに展開
my $ret = &common::param_check($kind, $pfile, \%argments);
if($ret != 0){
	exit(0);
}

#################################################################
# リポジトリ種別によりモジュールインクルード
#################################################################
if($kind eq "dspace"){
#	use dspace;
	require 'dspace.pm';
}
elsif ($kind eq "e-repository"){
#	use erepo.pm;
	require 'erepo.pm';
}
elsif ($kind eq "nalis"){
#	use nalis.pm;
	require 'nalis.pm';
}
else{
	print "[ERROR]リポジトリ種別に誤りがあります。\n";
	print "リポジトリ種別:dspace\n";
	print "               e-repository\n";
	print "               nalis\n";
	exit(0);
}

#----------------------------------------
# ログファイルのOPEN
#----------------------------------------
my $log_file = $argments{'log-file'};
if(!$log_file){
	# 未設定の場合はデフォルト値をセット
	$log_file = "./log/output2weko.log";
}
$ret = &common::log_open($log_file);
if($ret != 0){
	exit(0);
}

#################################################################
# NII資源タイプファイルを読み込み変数に展開
#################################################################
my %type_nii_list = ();	# NII資源タイプ格納用hash
# NII資源タイプを%type_nii_listに展開
&common::readfile($argments{'type-nii-list'},\%type_nii_list);

my $msg = "処理開始";
&common::log_write($common::LOG_INFO,$msg);

#----------------------------------------
# 出力データの容量の上限
#----------------------------------------
my $max_size = $argments{'max-size'};

#################################################################
# データベース接続
#################################################################
$ret = &RepositoryControl::db_connect(\%argments);
if($ret != 0){
	exit(0);
}

#################################################################
# 該当データ件数取得
#################################################################
my $data_num = &RepositoryControl::db_count(\%argments);
if($data_num < 0){
	# データベースエラー
	exit(0);
}
elsif ($data_num == 0){
	# 該当件数なし
	my $msg = "出力対象のデータがありませんでした。";
	&common::log_write($common::LOG_INFO,$msg);
	# データベース切断
	&RepositoryControl::db_final();
	exit(0);
}

#################################################################
# データ出力用tsvファイルの作成
#################################################################
my $outdir = $argments{'out-dir'};	# パラメタファイルの出力データ格納先
if($outdir eq ""){
	# 出力データ格納先が指定されていない場合はデフォルトをセット
	$outdir = "./outdata";
}
#----------------------------------------
# 出力データ格納先の存在チェック
#----------------------------------------
if(! -d $outdir){
	# 存在しない
	my $msg = "指定された出力データ格納先[$outdir]が存在しません。[$outdir]を作成し、再度実行してください。";
	&common::log_write($common::LOG_ERROR,$msg);
	# データベース切断
	&RepositoryControl::db_final();
	exit(0);
}
#----------------------------------------
# 出力データ格納先の書き込み権限チェック
#----------------------------------------
if(! -w $outdir){
	# 書き込み権限なし
	my $msg = "指定された出力データ格納先[$outdir]の書き込み権限がありません。";
	&common::log_write($common::LOG_ERROR,$msg);
	# データベース切断
	&RepositoryControl::db_final();
	exit(0);
}
#------------------------------------------
# トップレベルディレクトリ[item001]の作成
#------------------------------------------
my $topdir = $outdir;
$topdir =~ s/\/$//;		# 末尾の「/」削除
$topdir .= "/item001";
#------------------------------------------
# トップレベルディレクトリの存在チェック
#------------------------------------------
if(-d $topdir){
	# 既に存在する
	my $msg = "[$topdir]が既に存在します。[$outdir]内の[item001]ディレクトリを削除し、再度実行してください。";
	&common::log_write($common::LOG_ERROR,$msg);
	# データベース切断
	&RepositoryControl::db_final();
	exit(0);
}
#------------------------------------------
# ディレクトリ作成
#------------------------------------------
$! = "";
mkdir $topdir;		# トップレベルディレクトリの作成
if ($!) {
	# ディレクトリ作成エラー
	my $msg = "[$outdir]配下にディレクトリが作成できません。権限等を見直し再度実行してください。エラーコード:$!";
	&common::log_write($common::LOG_ERROR,$msg);
	# データベース切断
	&RepositoryControl::db_final();
	exit(0);
}
#------------------------------------------
# tsvファイル作成(BOM付きで作成)
#------------------------------------------
my $out_file = "$topdir/metadata.tsv";
unless(open(OUT, "> $out_file")){
	my $msg = "[$topdir]配下にファイルが作成できません。権限等を見直し再度実行してください。エラーコード:$!";
	&common::log_write($common::LOG_ERROR,$msg);
}
my $bom = "\x{feff}"; #BOM
utf8::encode($bom);
print OUT $bom;


#################################################################
# 処理対象データ件数表示
#################################################################
my $msg = "処理対象データは ". &common::convert($data_num) ." 件あります。";
&common::log_write($common::LOG_INFO,$msg);

#################################################################
# 1行目出力用データタグ取得
#################################################################
my $tags = &RepositoryControl::db_getTags();
#------------------------------------------
# 1行目出力
#------------------------------------------
print OUT "$tags\r\n";


#################################################################
# データ件数分Loopし1行ずつファイルに出力
#################################################################
my ($item_key,$line_data,$file_data,$file_description,$weko_item);
my $comm_array;
my $i;
my $display_count = $argments{'display-count'} + 0;
if($display_count == 0){
	# 未設定の場合はデフォルト値をセット
	$display_count = 1000;
}

for($i=0; $i < $data_num; $i++){

	#------------------------------------------
	# 1行分データ取得
	#------------------------------------------
	($item_key,$line_data,$weko_item,$comm_array) = &RepositoryControl::db_getData(\%argments,\%type_nii_list);
	#------------------------------------------
	# ファイル出力(１データ分＋アイテムキー)
	#------------------------------------------
	print OUT "$line_data";
	#------------------------------------------
	# ファイル出力(POS_INDEX) 区切り文字は「|」
	#------------------------------------------
	print OUT "\t".join("|",@$comm_array);
	#------------------------------------------
	# ファイル出力(アイテムキー)
	#------------------------------------------
	print OUT "\t$item_key";

	#------------------------------------------
	# 本文ファイル取得、ディレクトリ作成、コピー
	#------------------------------------------
	($file_data, $file_description) = &RepositoryControl::db_getFulltext($item_key, $topdir, \%argments);
	if($file_data == -1){
		# データベース切断
		&RepositoryControl::db_final();
		exit(0);
	}
	#------------------------------------------
	# ファイル出力(ファイル列)
	#------------------------------------------
	print OUT "\t".join($argments{'delimiter'},@$file_data);

	#------------------------------------------
	# ファイル出力(ファイル(説明)列)
	#------------------------------------------
	if($file_description){
		print OUT "\t".join($argments{'delimiter'},@$file_description);
	} else {
		print OUT "\t";
	}

	#------------------------------------------
	# WEKOアイテムタイプ出力
	#------------------------------------------
	if(!$weko_item){
		my $msg = "NII資源タイプが設定されていないか、もしくは設定値に誤りがあるためWEKOアイテムタイプを出力できませんでした。ITEM_KEY:".&common::convert($item_key);
		&common::log_write($common::LOG_WARNING,$msg);
	}
	print OUT "\t".$weko_item."\r\n";

	#------------------------------------------
	# POS_INDEX + ファイル名 長さチェック
	#------------------------------------------
	&leng_check($item_key,$comm_array,$file_data);

	#------------------------------------------
	# 途中経過
	#------------------------------------------
	if($display_count > 0){
		if(!(($i + 1) % $display_count)){
			my $msg = ($i + 1) . " / ".&common::convert($data_num)." 件処理しました。";
			&common::log_write($common::LOG_INFO,$msg);
		}
	}

	#------------------------------------------
	# 出力データの容量の上限の確認
	#------------------------------------------
	if($max_size ne ""){
		$ret = &common::size_check($outdir, $max_size);
		if($ret != 0){
			my $msg = "容量の上限（" . $max_size . "）を超えたため処理を打ち切ります。";
			&common::log_write($common::LOG_ERROR,$msg);
			# データベース切断
			&RepositoryControl::db_final();
			exit(0);
		}
	}
}

# 全件出力終了
my $msg = $i." / ".&common::convert($data_num)." 件処理しました。";
&common::log_write($common::LOG_INFO,$msg);

#################################################################
# データ出力用tsvファイルClose
#################################################################
close(OUT);

#################################################################
# eperson情報をファイルに出力
#################################################################
$ret = &RepositoryControl::db_getEperson($topdir);
if($ret == -1){
	# データベース切断
	&RepositoryControl::db_final();
	exit(0);
}

#################################################################
# データベース切断
#################################################################
&RepositoryControl::db_final();

#################################################################
# 出力データ圧縮処理
# (圧縮指定されている場合のみ行う)
#################################################################
if($argments{'compression'} eq "YES"){
	my $msg = "出力データを圧縮しています。";
	&common::log_write($common::LOG_INFO,$msg);
	my $sysret = system("cd $outdir ; tar zcf ./item001.tar.gz item001");
	if($sysret != 0){
		my $msg = "出力データの圧縮に失敗しました。エラーコード:$sysret";
		&common::log_write($common::LOG_ERROR,$msg);
		exit(0);
	}
}

my $msg = "処理終了";
&common::log_write($common::LOG_INFO,$msg);

exit(0);

#################################################################
# インデックスとファイル名の長さをShift-JISで計算しチェックする
#################################################################
sub leng_check {
	my $item_key = @_[0];
	my $comm_array = @_[1];
	my $file_data = @_[2];

	my $max_len = 200;

	my $length_comm = 0;
	my $length_fname = 0;

	foreach my $comm_name (@$comm_array){
		# インデックス名の長さ
		$comm_name = Jcode::convert( $comm_name , "sjis", "utf8" );
		$length_comm = length($comm_name);
		# インデックス名が既に制限長を超えている
		if( $length_comm > $max_len){
			my $msg = "ファイル名を含めたインデックス名が長すぎるため、SCfWでのロードが中断される可能性があります。ITEM_KEY:".&common::convert($item_key);
			&common::log_write($common::LOG_WARNING,$msg);
			return;
		}

		foreach my $file_name (@$file_data){
			# ファイル名の長さ
			$file_name = Jcode::convert( $file_name , "sjis", "utf8" );
			$length_fname = length($file_name);

			if( ($length_comm + $length_fname) > $max_len){
				my $msg = "ファイル名を含めたインデックス名が長すぎるため、SCfWでのロードが中断される可能性があります。ITEM_KEY:".&common::convert($item_key);
				&common::log_write($common::LOG_WARNING,$msg);
				return;
			}
		}
	}
}



